

---enumerate functions

--[[Usage:
for ped in EnumeratePeds() do
  <do something with 'ped'>
end
]]

local entityEnumerator = {
  __gc = function(enum)
    if enum.destructor and enum.handle then
      enum.destructor(enum.handle)
    end
    enum.destructor = nil
    enum.handle = nil
  end
}

local function EnumerateEntities(initFunc, moveFunc, disposeFunc)
  return coroutine.wrap(function()
    local iter, id = initFunc()
    if not id or id == 0 then
      disposeFunc(iter)
      return
    end
    
    local enum = {handle = iter, destructor = disposeFunc}
    setmetatable(enum, entityEnumerator)
    
    local next = true
    repeat
      coroutine.yield(id)
      next, id = moveFunc(iter)
    until not next
    
    enum.destructor, enum.handle = nil, nil
    disposeFunc(iter)
  end)
end

function EnumerateObjects()
  return EnumerateEntities(FindFirstObject, FindNextObject, EndFindObject)
end

function EnumeratePeds()
  return EnumerateEntities(FindFirstPed, FindNextPed, EndFindPed)
end

function EnumerateVehicles()
  return EnumerateEntities(FindFirstVehicle, FindNextVehicle, EndFindVehicle)
end

function EnumeratePickups()
  return EnumerateEntities(FindFirstPickup, FindNextPickup, EndFindPickup)
end

local listOn2 = false
function round(num, numDecimalPlaces)
  local mult = 10^(numDecimalPlaces or 0)
  return math.floor(num * mult + 0.5) / mult
end


function checkAndGetGroundZ(x, y, z,blFindGroundZ)
local groundFound = false
local groundCheckHeights = {-100.0, -50.0,0.0, 50.0, 100.0, 150.0, 200.0, 250.0, 300.0, 350.0, 400.0,450.0, 500.0, 550.0, 600.0, 650.0, 700.0, 750.0, 800.0}
local ground
			

	local unused,zGround = GetGroundZFor_3dCoord(x, y, z)
	--print("checkAndGetGroundZ: Groundz:"..zGround)
	--print("checkAndGetGroundZ:blFindGroundZ:"..tostring(blFindGroundZ)) 
	if zGround == 0.0 then 
		if blFindGroundZ then 
			
			for i,height in ipairs(groundCheckHeights) do
				RequestCollisionAtCoord(x, y, height)
				Wait(0) --< NEEDED?
				--SetEntityCoordsNoOffset(Ped, rXoffset,rYoffset,height, 0, 0, 1)
				ground,zGround = GetGroundZFor_3dCoord(x,y,height)
				if(ground) then
					zGround = zGround + 3
					groundFound = true
					break;
				end
			end
			if(not groundFound) then --or zGround < 0.0 then --zGround < 0.0 is water. turn off for boat missions
				print("not found ground")
				--print("checkAndGetGroundZ: findZoffset:"..zGround) 
				--print("checkAndGetGroundZ: groundFound:"..tostring(groundFound)) 
				zGround = 0.0 
			else 
			print("found ground")
				--print("checkAndGetGroundZ: findZoffset:"..zGround) 
				--print("checkAndGetGroundZ: groundFound:"..tostring(groundFound)) 
				
			end
		end
	end
	print("checkAndGetGroundZ: FinalGroundz:"..zGround) 
	return zGround

end

Citizen.CreateThread(function()
    listOn2 = false
    while true do
        Wait(0)

        if IsControlPressed(0, 27)--[[ INPUT_PHONE ]] then
            if not listOn2 then
				
				local pos = GetEntityCoords(GetPlayerPed(-1))
				local playerpos = "{ x = " .. round(pos.x,2) .. ", y = " .. round(pos.y,2) .. ", z = " .. round(pos.z,2) .. ", heading = "..round(GetGameplayCamRelativeHeading()+GetEntityHeading(GetPlayerPed(-1)),2) .." },\n"  --",playerheading = "..GetEntityHeading(GetPlayerPed(-1)) .." },\n" --GetGameplayCamRelativeHeading()+GetEntityHeading(GetPlayerPed(-1))
				local unused,zGround = GetGroundZFor_3dCoord(pos.x,pos.y,pos.z)
				--TriggerEvent('chatMessage', "Z coord"..round(pos.z,2).. ",GetGroundZ:"..zGround)
				TriggerEvent('chatMessage', "x:"..pos.x.. ",y:"..pos.y.."z:"..pos.z)
			    TriggerServerEvent("dumpposition", playerpos)
			--[[	
			vehiclehash = GetHashKey("buzzard")
			RequestModel(vehiclehash)
			RequestModel(randomPedModelHash)
			while not HasModelLoaded(vehiclehash) do
			  Wait(1)
			end
				
				CreateVehicle("buzzard", pos.x, pos.y,pos.z, 0, 1, 0)
                
				]]--
				
				--SetAudioFlag("LoadMPData", true);
				
				--fail:
				--PlaySoundFrontend(-1, "Crash", "DLC_HEIST_HACKING_SNAKE_SOUNDS", 1) 
				
				--PlaySoundFrontend(-1, "GO", "HUD_MINI_GAME_SOUNDSET", 1) 
				
				
				--PlaySoundFrontend(-1, "MP_5_SECOND_TIMER", "HUD_FRONTEND_DEFAULT_SOUNDSET", 1) 
				
				--PlaySoundFromEntity(GetSoundId(), "POLICE_CRASH", GetPlayerPed(-1), "PAPARAZZO_03A", 0, 0)
				--PlaySoundFromEntity(GetSoundId(),  "KISS", GetPlayerPed(-1), "ROAD_RACE_SOUNDSET", 1, 60) 
				
				--success:
				--PlaySoundFrontend(-1, "MP_RANK_UP", "HUD_FRONTEND_DEFAULT_SOUNDSET", 1)
				--PlaySoundFrontend(-1, "FIRST_PLACE", "HUD_MINI_GAME_SOUNDSET", 1)
				
				--******
				--PlaySoundFrontend(-1, "MEDAL_UP", "HUD_MINI_GAME_SOUNDSET", 1)
				
				--rescue:
				--PlaySoundFrontend(-1, "ROUND_ENDING_STINGER_CUSTOM", "CELEBRATION_SOUNDSET", 1)
				
				--local zGround = checkAndGetGroundZ(1800.0,100.0,0.0,true)
							
							--print("RandomLocation:zGround"..tostring(zGround))
							--local isWater = Citizen.InvokeNative( 0x8974647ED222EA5F,4200.0, 500.0, 1000.0,4200.0, 500.0,-500.0,Citizen.ReturnResultAnyway())
							
							--print(isWater.z)
							
							--if TestVerticalProbeAgainstAllWater(500.0,4200.0,100.0,208) == 1 then --TestVerticalProbeAgainstAllWater(500.0,4200.0,0.0,1,500) 
								--print("RandomLocation:FOUND WATER")
							--end
					--print("getwaterheight:"..tostring(GetWaterHeight(1800.0,100.0,zGround)))	
						--print(randomplace.x..","..randomplace.y..","..randomplace.z)
						--print("IsHost:"..NetworkIsHost())
							
				
				
				listOn2 = true
			   while listOn2 do
                    Wait(0)
                    if(IsControlPressed(0, 27) == false) then
                        listOn2 = false
  
                        break
                    end
                end
            end
        end
    end
end)

--[[
Citizen.CreateThread(function()
    while true do
        Wait(5000)
				--local pos = GetEntityCoords(GetPlayerPed(-1))
  				--local unused,zGround = GetGroundZFor_3dCoord(pos.x,pos.y,pos.z)
				--print('plot:'.."Z coord"..round(pos.z,2).. ",GetGroundZ:"..zGround)
				--TriggerEvent('chatMessage', "Z coord"..round(pos.z,2).. ",GetGroundZ:"..zGround)
				
				for ped in EnumeratePeds() do
						if DecorGetInt(ped, "mrpcheckspawn") > 0   then
							
							--check if GroundZ is zero for them still 
							local coords = GetEntityCoords(ped)
							local unused,zGround = GetGroundZFor_3dCoord(coords.x,coords.y,coords.z)
							print('LOOP: COORDS X:'..coords.x..",Y:"..coords.y..",Z:"..coords.z)
							--print('LOOP:unused '..testpedcnt..':'..tostring(unused))
							print('LOOP:rZoffset :'..zGround)
							
							--local pos = GetEntityCoords(GetPlayerPed(-1))
							--local unused2,zGround2 = GetGroundZFor_3dCoord(pos.x,pos.y,pos.z)
							--print('MRPplot:'.."Z coord"..round(pos.z,2).. ",GetGroundZ:"..zGround2)			
						end
				end
				
				
				
    end
end)
]]--